"""
A Python package to interpolate data from a NetCFD file onto a VTU file.

    **netcdf2vtu** provides a the *Mapper* class to read in a NetCFD input
    file, a destination VTU file and to set some specifics for data
    interpolation.

"""

from netCDF4 import Dataset
import numpy as np
from pyproj import Transformer
import vtk
from vtk.util import numpy_support


class Mapper(object):
    """
    A class for interpolating data from a NetCFD file on a VTU file.

    Parameters
    ----------
    nc_path : str
        The path to the NetCFD input file.
    vtu_path : str
        The path to the destination VTU file.
    data_var_names : list of str
        A list of variable names to read from the NetCFD file.
    map_func_type : int
        The type of mapping function to use for interpolation.
    nc_crs : str
        The coordinate reference system of the NetCFD file.
    vtu_crs : str
        The coordinate reference system of the VTU file.
    nullvalue : float
        The value to use for missing data points during interpolation.
    kwargs_nc : dict
        Additional keyword arguments to pass to the NetCFD4 library when
         reading the NetCFD file.

    """

    def __init__(
        self,
        nc_path,
        vtu_path,
        data_var_names,
        map_func_type,
        nc_crs,
        vtu_crs,
        nullvalue,
        **kwargs_nc
    ):

        self.nc = dict(path = nc_path,
                        crs = nc_crs,
                        data_names = data_var_names)
        self.in_vtu = dict(path = vtu_path,
                        crs = vtu_crs)

        self.map_func_type = map_func_type
        self.nullvalue = nullvalue

        # read source nc
        dset = Dataset(self.nc["path"], mode = "r", format = "NETCDF4",
                        **kwargs_nc)
        self.nc.update({"dataset" : dset})

        # read dst vtu
        dst_vtu = read_vtu(self.in_vtu["path"])
        self.in_vtu.update({"vtu" : dst_vtu})


    def get_nc_variables(self):
        """
        Returns a numpy.ndarray variable data of the NetCFD dataset.
        """
        self.nc["dataset"].variables


    def set_nc_coord_names(self,
                            lat_name,
                            lon_name,
                            time_name = None):
        """
        Sets the names of the latitude, longitude, and time variables
        to be read from the dataset in `self.nc["dataset"]`.
        """
        self.nc.update({"coord_names" : {"lat_name" : lat_name,
                                        "lon_name" : lon_name,
                                        "time_name" : time_name}})


    def set_nc_data_names(self, var_names):
        """
        Sets the names of the variables to be read from the NetCFD
        dataset in `self.nc["dataset"]`.

        Parameters
        ----------
        var_names : list of str
            A list of variable names to read from the NetCFD file.
        """
        self.nc["data_names"] = var_names


    def read_nc_coords(self):
        """
        Reads the latitude, longitude, and time coordinates from the
        NetCFD dataset in `self.nc["dataset"]`.

        """
        lat_name, lon_name, time_name = self.nc["coord_names"].values()

        lat_dat = self.nc["dataset"].variables[lat_name][:].filled()
        lon_dat = self.nc["dataset"].variables[lon_name][:].filled()

        if time_name is not None:
            time = self.nc["dataset"].variables[time_name][:].filled()
        else:
            time = None

        coords = {"lat": lat_dat, "lon": lon_dat, "time" : time}
        self.nc.update({"coords" : coords})


    def read_nc_data(self):
        """
        Reads the data variables specified in `self.nc["data_names"]`
        from the NetCFD dataset.

        """
        nc_data = get_nc_data(self.nc["dataset"],
                                  self.nc["data_names"])
        self.nc.update({"data" : nc_data})


    def interpolate(self):
        """
        Interpolates the data from the NetCFD file onto the VTU file.
        It creates a vtkPolyData instance from `self.nc["dataset"]` and
        interpolates this on a copy of an instance of
        vtkUnstructuredGrid created from the input VTU file.

        """
        self.vtp = create_vtp(
                    self.nc["coords"]["lon"],
                    self.nc["coords"]["lat"],
                    self.nc["crs"],
                    self.in_vtu["crs"])

        nc_data_to_vtp(self.vtp,
                           self.nc["data"],
                           self.nc["coords"]["time"])

        print("NetCDF converted to VTP.")


        self.out_vtu = interpolate_vtp_data_on_vtu(
                        self.vtp,
                        self.in_vtu["vtu"],
                        self.map_func_type,
                        self.nullvalue)

        print("Data from VTP interpolated to VTU.")


    def write_out_vtu(self, path):
        """
        Writes the updated instance of vtkUnstructuredGrid to the
        specified path as VTU file.

        Parameters
        ----------
        path : str
            The path of the VTU file to be written.

        """
        write_vtu(self.out_vtu, path)
        print("New VTU mesh written to disk.")


    def write_vtp(self, path):
        """
        Writes the vtkPolyData instance created to the
        specified path as VTP file.

        Parameters
        ----------
        path : str
            The path of the VTP file to be written.

        """
        write_vtp(self.vtp, path)


    def map(self,
            out_path,
            lat_name,
            lon_name,
            time_name = None):
        """
        A wrapper for simple interpolation.
        """
        self.set_nc_coord_names(lat_name,
                                lon_name,
                                time_name)
        self.read_nc_coords()
        self.read_nc_data()
        self.interpolate()
        self.write_out_vtu(out_path)



def get_nc_data(nc, data_var_names):
    """
    Extract variable data out of a netcdf4 dataset.

    Parameters
    ----------
    nc : netCDF4.Dataset
        The NetCFD dataset to extract data from.
    data_var_names : list of str
        A list of variable names to extract from nc.

    Returns
    -------
    list
        A list of tuples, where each tuple contains a variable name and the corresponding data as a numpy array.

    """
    vars_data = []
    for i, var_name in enumerate(data_var_names):
        t = (var_name, nc.variables[var_name][:].filled())
        vars_data.append(t)

    return(vars_data)


def create_vtp(lon_dat, lat_dat, nc_crs, vtu_crs):
    """
    Initializes a vtkPolyData object and sets its points and cells based
     on input latitude and longitude coordinates.
    The point coordinates are transformed to the specified destination
    coordinate reference system.

    initialize src_poly as vtkPolyData
    set points and cells for src_poly (vtkPolyData object where NetCFD
    data goes into)
    cells are vertice cells based on points
    point coordinates are transformed

    Parameters
    ----------
    lon_dat : numpy.ndarray
        A numpy array of longitude coordinates.
    lat_dat : numpy.ndarray
        A numpy array of latitude coordinates.
    nc_crs : str
        The coordinate reference system of the input latitude and
        longitude coordinates.
    vtu_crs : str
        The destination coordinate reference system to transform the
        point coordinates to.

    Returns
    -------
    vtk.vtkPolyData
        A vtkPolyData object with points and cells set based on input
        coordinates.

    """
    # check dims and def point array
    if len(lon_dat.shape) == 1 and len(lat_dat.shape) == 1:

        xv, yv, zv = np.meshgrid(lon_dat, lat_dat, np.array([0]))
        xv = xv.ravel()
        yv = yv.ravel()
        zv = zv.ravel()

    elif len(lon_dat.shape) == 2 and len(lat_dat.shape) == 2:

        xv = lon_dat.ravel()
        yv = lat_dat.ravel()
        zv = np.repeat(0, len(xv))

    else:
        raise ValueError("Shape of coordinate arrays not as expected!")

    points = np.array([xv, yv, zv]).transpose()

    # def vtkPoints/cells
    nc_points = vtk.vtkPoints()
    nc_cells = vtk.vtkCellArray()

    # transform points coordiantes
    transf = Transformer.from_crs(
                            nc_crs,
                            vtu_crs,
                            always_xy=True)

    for i in range(len(points)):
        p = transf.transform(points[i][0], points[i][1])
        ind = nc_points.InsertNextPoint(p[0], p[1], 0)
        nc_cells.InsertNextCell(1)
        nc_cells.InsertCellPoint(ind)

    # define vtkPolydata obj
    vtp = vtk.vtkPolyData()
    vtp.SetPoints(nc_points)
    vtp.SetVerts(nc_cells)
    return(vtp)


def nc_data_to_vtp(vtp, nc_data, time = None):
    """
    Adds extracted variable data from a NetCFD file to a vtkPolyData
    object.

    Parameters
    ----------
    vtp : vtk.vtkPolyData
        The vtkPolyData object to add the NetCFD data to.
    nc_data : list
        A list of tuples, where each tuple contains a variable name and its corresponding data as a numpy array.
    time : numpy.ndarray, optional
        An array of time values for the NetCFD data. If not provided,
        the time value will be set to "FALSE".

    """
    if time is None:
        time = "F"

    for j,val in enumerate(nc_data):
        for i in range(len(time)):

            if type(time) is str:
                arr_name = val[0]
            else:
                arr_name = val[0] + "_%s" % str(int(time[i]))

            new_point_arr_vtk = numpy_support.numpy_to_vtk(val[1][i].ravel())
            new_point_arr_vtk.SetName(arr_name)
            vtp.GetPointData().AddArray(new_point_arr_vtk)

    vtp.Modified()



def write_vtp(vtp, outputfile_name):
    """
    Writes a vtkPolyData object to a file.

    Parameters
    ----------
    vtp : vtk.vtkPolyData
        The vtkPolyData object to write to a file.
    outputfile_name : str
        The path of the file to write the vtkPolyData object to.

    """
    write_ouput = vtk.vtkXMLPolyDataWriter()
    write_ouput.SetInputData(vtp)
    write_ouput.SetFileName(outputfile_name)
    write_ouput.Write()


def read_vtu(in_filepath):
    """
    Reads an OGS mesh file and returns it as a vtkUnstructuredGrid object.

    Parameters
    ----------
    in_filepath : str
        The path of the VTU file to read.

    Returns
    -------
    vtk.vtkUnstructuredGrid
        The VTU file as a vtkUnstructuredGrid instance.

    """
    dst = vtk.vtkXMLUnstructuredGridReader()
    dst.SetFileName(in_filepath)
    dst.Update()
    return(dst.GetOutput())


def interpolate_vtp_data_on_vtu(vtp, vtu, map_func_type, nullvalue):
    """
    Maps the data of a vtkPolyData object onto a vtkUnstructuredGrid
    object using a specified interpolation algorithm.

    Parameters
    ----------
    vtp : vtk.vtkPolyData
        The object that contains the data to be interpolated.
    vtu : vtk.vtkUnstructuredGrid
        The object that the data will be interpolated onto.
    map_func_type : int
        The type of mapping function to use for interpolation.
    nullvalue : float
        The value to use for missing data points during interpolation.

    Returns
    -------
    vtk.vtkUnstructuredGrid
        An object with the interpolated data.

    """
    interpolator = vtk.vtkPointInterpolator()
    interpolator.SetInputData(vtu)
    interpolator.SetSourceData(vtp)
    # def interpolation algorithm
    interpolator.SetKernel(set_map_function(map_func_type))
    # def value if interpolation does not work
    interpolator.SetNullValue(nullvalue)
    interpolator.Update()
    return(interpolator.GetOutput())



def write_vtu(vtu, path):
    """
    Writes a vtkUnstructuredGrid  object to a file.

    Parameters
    ----------
    vtp : vtk.vtkUnstructuredGrid
        The vtkUnstructuredGrid object to write to a file.
    outputfile_name : str
        The path of the VTU file to be written.

    """
    write_output = vtk.vtkXMLUnstructuredGridWriter()
    write_output.SetFileName(path)
    write_output.SetInputData(vtu)
    write_output.Write()


def set_map_function(map_func_type):
    """
    Returns the specified interpolation/mapping algorithm.

    Parameters
    ----------
    map_func_type : int
        The type of mapping function to use for interpolation.
        1: voronoi_kernel
        2: gaussian_kernel
        3: shepard_kernel

    Returns
    -------
    vtk.vtkInterpolationKernel
        An object of the specified interpolation/mapping algorithm.

    """
    switcher = {
        1: voronoi_kernel,
        2: gaussian_kernel,
        3: shepard_kernel
    }
    # Get the function from switcher dictionary
    func = switcher.get(map_func_type, lambda: "Invalid func_type")
    return(func())


def gaussian_kernel():
    """
    Gaussian filter of point cloud defined by
    radius around a point.

    Returns
    -------
    vtk.vtkGaussianKernel
        The Gaussian filter interpolation kernel.

    """
    int_kernel = vtk.vtkGaussianKernel()
    int_kernel.SetSharpness(2)
    int_kernel.SetRadius(4000)
    #TODO: add kwargs to set the sharpness and radius
    return(int_kernel)


def voronoi_kernel():
    """
    Preferred for categorical data.
    Takes value of the closest point.

    Returns
    -------
    vtk.vtkVoronoiKernel
        The voronoi interpolation kernel.

    """
    int_kernel = vtk.vtkVoronoiKernel()
    return(int_kernel)


def shepard_kernel():
    """
    Interpolation of a point cloud object defined by
    power function of the radius around a point.

    Returns
    -------
    vtk.vtkShepardKernel
        The Shepard interpolation kernel.

    """
    int_kernel = vtk.vtkShepardKernel()
    int_kernel.SetPowerParameter(2)
    int_kernel.SetRadius(4000)
    return(int_kernel)


# -*- coding: utf-8 -*-
"""These are the unit tests of the netcdf2vtu module."""

import os
from pytest import raises
import subprocess
import shlex

from netCDF4 import Dataset
from numpy import array, ndarray
from numpy.testing import assert_array_almost_equal
import vtk
from vtkmodules.vtkCommonDataModel import vtkUnstructuredGrid, vtkPolyData
from vtkmodules.vtkFiltersPoints import vtkVoronoiKernel, vtkGaussianKernel, vtkShepardKernel
from vtk.numpy_interface import dataset_adapter as dsa

import netcdf2vtu.netcdf2vtu as n2v


nc_file1 = "data/ex1_3.nc"
nc_file2 = "data/ex2_4.nc"

in_vtu_path = "data/ogs.vtu"
out_vtu_path_mapper = "temp_mapper.vtu"
out_vtp_path_mapper = "temp_mapper.vtp"
out_vtu_path_fun = "temp_fun.vtu"
out_vtp_path_fun =  "temp_fun.vtp"
out_vtu_path_cli = "temp_cli.vtu"
ref_vtp_path = "tests/ref.vtp"
ref_vtu_path = "tests/ref.vtu"
data_var_names = ["SWC_L01", "SWC_L02"]
map_func_type = 1
nc_crs = "EPSG:4326"
vtu_crs = "EPSG:5684"
nullvalue = -9999
coordnames = ["lat", "lon", "time"]

class TestMapper():

    def setup_method(self):
        self.nc_file = nc_file1
        self.nc_data_var = data_var_names
        self.vtu_in = in_vtu_path
        self.vtu_out = out_vtu_path_mapper
        self.vtp_out = out_vtp_path_mapper
        self.nc_crs = nc_crs
        self.vtu_crs = vtu_crs
        self.map_func_type = map_func_type
        self.nullval = nullvalue
        self.coordnames = coordnames

        self.dataset = Dataset(self.nc_file, mode = "r", format = "NETCDF4")
        self.vtu = n2v.read_vtu(self.vtu_in)

        self.mapper = n2v.Mapper(
            nc_path = self.nc_file,
            vtu_path = self.vtu_in,
            data_var_names = self.nc_data_var,
            map_func_type = self.map_func_type,
            nc_crs = self.nc_crs,
            vtu_crs = self.vtu_crs,
            nullvalue = self.nullval
        )

        self.lat_dat = self.dataset.variables[self.coordnames[0]][:].filled()
        self.lon_dat = self.dataset.variables[self.coordnames[1]][:].filled()
        self.time_dat = self.dataset.variables[self.coordnames[2]][:].filled()

        self.nc_data = n2v.get_nc_data(self.dataset, self.nc_data_var)


    def test_init_mapper(self):

        mapper = n2v.Mapper(
            nc_path = self.nc_file,
            vtu_path = self.vtu_in,
            data_var_names = self.nc_data_var,
            map_func_type = self.map_func_type,
            nc_crs = self.nc_crs,
            vtu_crs = self.vtu_crs,
            nullvalue = self.nullval
        )

        assert type(mapper) is n2v.Mapper

        assert mapper.nc["path"] == self.nc_file
        assert mapper.nc["crs"] == self.nc_crs
        assert mapper.nc["data_names"] == self.nc_data_var
        assert type(mapper.nc["dataset"]) == type(self.dataset)

        assert mapper.in_vtu["path"] == self.vtu_in
        assert mapper.in_vtu["crs"] == self.vtu_crs
        assert type(mapper.in_vtu["vtu"]) == type(self.vtu)

        assert mapper.map_func_type == self.map_func_type
        assert mapper.nullvalue == self.nullval


    def test_map(self, tmp_path):

        a, b, c = self.coordnames
        vtu_path = str(tmp_path/self.vtu_out)
        vtp_path = str(tmp_path/self.vtp_out)

        self.mapper.map(vtu_path, a, b, c)
        self.mapper.write_vtp(vtp_path)

        # check variable names
        assert isinstance(self.mapper.nc["coord_names"], dict)
        assert self.mapper.nc["coord_names"]["lat_name"] == self.coordnames[0]
        assert self.mapper.nc["coord_names"]["lon_name"] == self.coordnames[1]
        assert self.mapper.nc["coord_names"]["time_name"] == self.coordnames[2]

        assert self.mapper.nc["data_names"] == self.nc_data_var

        # check coordinate data
        self.mapper.nc["coords"]["lat"] = self.lat_dat
        self.mapper.nc["coords"]["lon"] = self.lon_dat
        self.mapper.nc["coords"]["time"] = self.time_dat

        # check extracted data
        assert_array_almost_equal(
            self.mapper.nc["data"][0][1],
            self.nc_data[0][1]
        )

        assert_array_almost_equal(
            self.mapper.nc["data"][1][1],
            self.nc_data[1][1]
        )

        # check created vtp and vtu
        isinstance(self.mapper.vtp, vtkPolyData)
        isinstance(self.mapper.out_vtu, vtkUnstructuredGrid)

        # check written files
        os.path.isfile(vtu_path)
        os.path.isfile(vtp_path)


class TestFunctions():

    def setup_method(self):
        # specifictaion for input and output files
        self.nc_file = nc_file1
        self.nc_data_var = data_var_names
        self.vtu_in = in_vtu_path
        self.vtu_out = out_vtu_path_fun
        self.vtu_ref_path = ref_vtu_path
        self.vtp_ref = ref_vtp_path
        self.vtp_out = out_vtp_path_fun
        self.nc_crs = nc_crs
        self.vtu_crs = vtu_crs
        self.map_func_type = map_func_type
        self.nullval = nullvalue
        # read reference netcdf file and extract data
        self.dataset = Dataset(self.nc_file, mode = "r", format = "NETCDF4")
        self.swcl01 = self.dataset.variables["SWC_L01"][:].filled()
        self.swcl02 = self.dataset.variables["SWC_L02"][:].filled()
        self.coordnames = coordnames
        self.lat_dat = self.dataset.variables[self.coordnames[0]][:].filled()
        self.lon_dat = self.dataset.variables[self.coordnames[1]][:].filled()
        self.time_dat = self.dataset.variables[self.coordnames[2]][:].filled()
        # create reference vtp
        rdr = vtk.vtkXMLPolyDataReader()
        rdr.SetFileName(self.vtp_ref)
        rdr.Update()
        self.ref_vtp = rdr.GetOutput()
        self.ref_vtp_dsa = dsa.WrapDataObject(self.ref_vtp)
        # read reference vtu
        dst = vtk.vtkXMLUnstructuredGridReader()
        dst.SetFileName(self.vtu_ref_path)
        dst.Update()
        self.ref_vtu = dst.GetOutput()
        self.ref_vtu_dsa = dsa.WrapDataObject(self.ref_vtu)

    def test_get_nc_data(self):

        l = n2v.get_nc_data(self.dataset, self.nc_data_var)

        isinstance(l, list)
        assert len(l) == 2
        isinstance(l[0], tuple)
        isinstance(l[1], tuple)
        assert l[0][0] == "SWC_L01"
        assert l[1][0] == "SWC_L02"
        assert l[0][1].shape == (2,27,21)
        assert l[1][1].shape == (2,27,21)

        # assert data
        assert_array_almost_equal(l[0][1][0], self.swcl01[0])
        assert_array_almost_equal(l[0][1][1], self.swcl01[1])
        assert_array_almost_equal(l[1][1][0], self.swcl02[0])
        assert_array_almost_equal(l[1][1][1], self.swcl02[1])


    def test_create_vtp_and_nc_data_to_vtp(self):

        ref_vtp_dsa = self.ref_vtp_dsa
        ref_vtp_points = ref_vtp_dsa.Points

        vtp = n2v.create_vtp(self.lon_dat, self.lat_dat, self.nc_crs, self.vtu_crs)

        isinstance(vtp, vtkPolyData)

        vtp_dsa = dsa.WrapDataObject(vtp)
        vtp_points = vtp_dsa.Points

        # compare coordinate points
        assert_array_almost_equal(
            array(vtp_points),
            array(ref_vtp_points))


    def test_nc_data_to_vtp(self):

        ref_vtp_dsa = self.ref_vtp_dsa

        # create vtp and add data
        vtp = n2v.create_vtp(self.lon_dat, self.lat_dat, self.nc_crs, self.vtu_crs)

        l = [("SWC_L01", self.swcl01),
             ("SWC_L02", self.swcl02)]
        n2v.nc_data_to_vtp(vtp, l, self.time_dat)

        vtp_dsa = dsa.WrapDataObject(vtp)

        # compare data with ref_vtp_dsa
        assert ref_vtp_dsa.PointData.keys() == vtp_dsa.PointData.keys()

        assert_array_almost_equal(
            ref_vtp_dsa.PointData.GetArray(0),
            vtp_dsa.PointData.GetArray(0))

        assert_array_almost_equal(
            ref_vtp_dsa.PointData.GetArray(1),
            vtp_dsa.PointData.GetArray(1))

        assert_array_almost_equal(
            ref_vtp_dsa.PointData.GetArray(2),
            vtp_dsa.PointData.GetArray(2))

        assert_array_almost_equal(
            ref_vtp_dsa.PointData.GetArray(3),
            vtp_dsa.PointData.GetArray(3))


    def test_interpolate_vtp_data_on_vtu(self):

        ref_vtu_dsa = self.ref_vtu_dsa

         # create vtp and add data
        vtp = n2v.create_vtp(self.lon_dat, self.lat_dat, self.nc_crs, self.vtu_crs)

        l = [("SWC_L01", self.swcl01),
             ("SWC_L02", self.swcl02)]
        n2v.nc_data_to_vtp(vtp, l, self.time_dat)

        # read vtu
        vtu = n2v.read_vtu(self.vtu_in)

        # interpolate
        out_vtu = n2v.interpolate_vtp_data_on_vtu(vtp, vtu, self.map_func_type, self.nullval)
        out_vtu_dsa = dsa.WrapDataObject(out_vtu)

        # compare out_vtu with ref_vtu
        # compare coordinate points
        assert_array_almost_equal(
            array(ref_vtu_dsa.Points),
            array(out_vtu_dsa.Points))
        # compare data
        assert ref_vtu_dsa.PointData.keys() == out_vtu_dsa.PointData.keys()

        assert_array_almost_equal(
            ref_vtu_dsa.PointData.GetArray(0),
            out_vtu_dsa.PointData.GetArray(0))

        assert_array_almost_equal(
            ref_vtu_dsa.PointData.GetArray(1),
            out_vtu_dsa.PointData.GetArray(1))

        assert_array_almost_equal(
            ref_vtu_dsa.PointData.GetArray(2),
            out_vtu_dsa.PointData.GetArray(2))

        assert_array_almost_equal(
            ref_vtu_dsa.PointData.GetArray(3),
            out_vtu_dsa.PointData.GetArray(3))


    def test_read_vtu(self):

        vtu = n2v.read_vtu(self.vtu_ref_path)
        isinstance(vtu, vtkUnstructuredGrid)


    def test_write_vtu(self, tmp_path):

        file = str(tmp_path/self.vtu_out)
        n2v.write_vtu(self.ref_vtu, file)
        os.path.isfile(file)
        # reload and test
        vtu = n2v.read_vtu(self.vtu_ref_path)
        isinstance(vtu, vtkUnstructuredGrid)


    def test_write_vtp(self, tmp_path):

        file = str(tmp_path/self.vtp_out)
        n2v.write_vtp(self.ref_vtp, file)
        os.path.isfile(file)
        # reload and test
        rdr = vtk.vtkXMLPolyDataReader()
        rdr.SetFileName(file)
        rdr.Update()
        vtp = rdr.GetOutput()
        isinstance(vtp, vtkPolyData)


    def test_set_map_function(self):

        map_fun1 = n2v.set_map_function(1)
        map_fun2 = n2v.set_map_function(2)
        map_fun3 = n2v.set_map_function(3)

        isinstance(map_fun1, vtkVoronoiKernel)
        isinstance(map_fun2, vtkGaussianKernel)
        isinstance(map_fun3, vtkShepardKernel)


class TestCli():

    def setup_method(self):
        # def command for cli
        self.com = ['netcdf2vtu', '-o', out_vtu_path_cli,
                    '-m', str(map_func_type), '-n', str(nullvalue),
                    '--lat', coordnames[0], '--lon', coordnames[1],
                    '--time', coordnames[2],
                    nc_file1, in_vtu_path, nc_crs, vtu_crs,
                    data_var_names[0]]


    def test_successful_exec(self, tmp_path):

        com = self.com[:]
        file = str(tmp_path/out_vtu_path_cli)
        com[2] = file
        result = subprocess.run(com, stdout=subprocess.PIPE)
        assert result.returncode == 0
        assert os.path.isfile(file)


    def test_missing_required_args(self):

        with raises(subprocess.CalledProcessError):
            subprocess.run(["netcdf2vtu"], stdout=subprocess.PIPE,
            check=True)


    def test_default_vals(self):

        com = self.com[:1]+self.com[11:]
        result = subprocess.run(com, stdout=subprocess.PIPE)
        assert result.returncode == 0
        assert os.path.isfile("out.vtu")
        os.remove("out.vtu")



    def test_invalid_arg_vals(self):

        com = self.com[:1]+["-m", "X"]+self.com[11:]
        with raises(subprocess.CalledProcessError):
            subprocess.run(com, stdout=subprocess.PIPE, check=True)


    def test_opt_args_not_given(self, tmp_path):

        com = self.com[:]
        file = str(tmp_path/"5_")+out_vtu_path_cli
        com[2] = file
        com.pop(11)
        com.pop(11)
        result = subprocess.run(com, stdout=subprocess.PIPE)
        assert result.returncode == 0
        assert os.path.isfile(file)




"""Command line interface for netcdf2vtu."""

import argparse

from .netcdf2vtu import Mapper

def main():

    # Define parser
    parser = argparse.ArgumentParser(
        prog="netcdf2vtu",
        description='Interpolates data from  netCDF4 files to VTU files.')

    parser.add_argument('-o', '--out_vtu', type=str, default="out.vtu",
                        help='Path to the output VTU file; default is '\
                            '"out.vtu".')
    parser.add_argument('-m', '--map_type', type=int, default=1,
                        help='Type of interpolation function; default is'\
                            ' 1.')
    parser.add_argument('-n', '--nullvalue', type=int, default=-9999,
                        help='Nullvalue; default is -9999.')
    parser.add_argument('--lat', type=str, default="lat",
                        help='Name of the latitude variable; default' \
                            ' is "lat".')
    parser.add_argument('--lon', type=str, default="lon",
                        help='Name of the longitude variable; default' \
                            ' is "lon".')
    parser.add_argument('--time', type=str,
                        help='Name of the time variable if present.')
    parser.add_argument('in_nc', type=str,
                        help='Path to the input netCDF file.')
    parser.add_argument('in_vtu', type=str,
                        help='Path to the input VTU file.')
    parser.add_argument('nc_crs', type=str,
                        help='CRS of input netcdf file')
    parser.add_argument('vtu_crs', type=str,
                        help='CRS of input vtu file.')
    parser.add_argument('var_names', nargs="+", type= str,
                        help='Names of the data variables in the input netcdf file.')

    args = parser.parse_args()

    # Assign args
    in_nc = args.in_nc
    in_vtu = args.in_vtu
    out_vtu = args.out_vtu
    var_names = args.var_names
    map_type = args.map_type
    nc_crs = args.nc_crs
    vtu_crs = args.vtu_crs
    nullval = args.nullvalue
    lat_name = args.lat
    lon_name = args.lon
    time_name = args.time

    # Do the interpolation
    mapper = Mapper(in_nc,
                    in_vtu,
                    var_names,
                    map_type,
                    nc_crs,
                    vtu_crs,
                    nullval)

    mapper.map(out_path = out_vtu,
                lat_name = lat_name,
                lon_name = lon_name,
                time_name = time_name)

if __name__ == "__main__":
    main()